@@ -18,50 +18,74 @@ import logging
 import warnings
 import time
 
-from .metadata_normalizer import MetadataNormalizer
-
 import pandas as pd
 
+from google.datacatalog_connectors.rdbms.scrape import \
+    base_metadata_enricher
+
+from google.datacatalog_connectors.rdbms.scrape.metadata_normalizer \
+    import MetadataNormalizer
+
+from google.datacatalog_connectors.rdbms.scrape.sql_objects \
+    import SQLObjectsMetadataScraper
+
+from google.datacatalog_connectors.rdbms.scrape import constants
+
 
 class MetadataScraper:
 
     def __init__(self):
-        pass
+        self.__sql_objects_scraper = SQLObjectsMetadataScraper(self)
 
-    def get_metadata(self,
-                     metadata_definition,
-                     connection_args=None,
-                     query=None,
-                     csv_path=None,
-                     user_config=None):
-        dataframe = self._get_metadata_as_dataframe(metadata_definition,
-                                                    connection_args, query,
-                                                    csv_path, user_config)
+    def scrape(self,
+               metadata_definition,
+               connection_args=None,
+               query=None,
+               csv_path=None,
+               config=None):
+        query = self.__resolve_base_metadata_query(config, query)
 
-        return MetadataNormalizer.to_metadata_dict(dataframe,
-                                                   metadata_definition)
+        dataframe = self.get_metadata_as_dataframe(metadata_definition,
+                                                   connection_args, query,
+                                                   csv_path, config)
 
-    def _get_metadata_as_dataframe(self,
-                                   metadata_definition,
-                                   connection_args=None,
-                                   query=None,
-                                   csv_path=None,
-                                   user_config=None):
+        base_metadata = MetadataNormalizer.normalize(dataframe,
+                                                     metadata_definition)
+        # CSV ingestion does not support SQL Objects
+        # so we return early.
+        if csv_path:
+            return base_metadata
+
+        sql_objects_metadata = self.__sql_objects_scraper.scrape(
+            config, connection_args)
+
+        if sql_objects_metadata:
+            base_metadata[constants.SQL_OBJECTS_KEY] = sql_objects_metadata
+
+        return base_metadata
+
+    def get_metadata_as_dataframe(self,
+                                  metadata_definition,
+                                  connection_args=None,
+                                  query=None,
+                                  csv_path=None,
+                                  config=None):
         if csv_path:
             logging.info('Scrapping metadata from csv path: "%s"', csv_path)
             dataframe = self._get_metadata_from_csv(csv_path)
-        elif connection_args and len(connection_args.keys()) > 0:
-            logging.info('Scrapping basic metadata from connection_args')
+        elif self._is_metadata_from_connection(connection_args):
+            logging.info('Scrapping metadata from connection_args')
             dataframe = self._get_base_metadata_from_rdbms_connection(
                 connection_args, query)
         else:
             raise Exception('Must supply either connection_args or csv_path')
 
-        if user_config:
+        if config:
             logging.info('Scrapping additional metadata from connection_args,'
                          'if configured')
-            dataframe = self._enrich_metadata_based_on_user_config(
-                user_config, dataframe, connection_args, metadata_definition)
+            dataframe = self._enrich_metadata_based_on_config(
+                config, dataframe, connection_args, metadata_definition,
+                csv_path)
 
         return dataframe
 
@@ -90,36 +114,39 @@ class MetadataScraper:
     def _create_dataframe(self, rows):
         return pd.DataFrame(rows)
 
-    def _enrich_metadata_based_on_user_config(self, user_config,
-                                              base_dataframe, connection_args,
-                                              metadata_definition):
+    def _enrich_metadata_based_on_config(self, config, base_dataframe,
+                                         connection_args, metadata_definition,
+                                         csv_path):
         enriched_dataframe = base_dataframe
 
-        if user_config.refresh_metadata_tables:
-            query_assembler = self._get_query_assembler()
-            exact_table_names = MetadataNormalizer.\
-                get_exact_table_names_from_dataframe(
-                    base_dataframe, metadata_definition)
-            refresh_queries = query_assembler.get_refresh_metadata_queries(
-                exact_table_names)
-            logging.info('Refreshing metadata')
-            self._refresh_metadata_from_rdbms_connection(
-                connection_args, refresh_queries)
+        # If the execution comes from CSV source,
+        # ignore the additional queries.
+        if not csv_path:
+            if config.refresh_metadata_tables:
+                query_assembler = self._get_query_assembler()
+                exact_table_names = MetadataNormalizer.\
+                    get_exact_table_names_from_dataframe(
+                        base_dataframe, metadata_definition)
+                refresh_queries = query_assembler.get_refresh_metadata_queries(
+                    exact_table_names)
+                logging.info('Refreshing metadata')
+                self._refresh_metadata_from_rdbms_connection(
+                    connection_args, refresh_queries)
 
-        if user_config.scrape_optional_metadata:
-            query_assembler = self._get_query_assembler()
-            optional_metadata = user_config.get_chosen_metadata_options()
-            optional_queries = query_assembler.get_optional_queries(
-                optional_metadata)
-            logging.info(
-                'Scraping metadata according to configuration file: {}'.format(
-                    optional_metadata))
-            enriched_dataframe = \
-                self._get_optional_metadata_from_rdbms_connection(
-                    connection_args, optional_queries, base_dataframe,
-                    metadata_definition)
+            if config.scrape_optional_metadata:
+                query_assembler = self._get_query_assembler()
+                optional_metadata = config.get_chosen_metadata_options()
+                optional_queries = query_assembler.get_optional_queries(
+                    optional_metadata)
+                logging.info(
+                    'Scraping metadata according to configuration file: {}'.
+                    format(optional_metadata))
+                enriched_dataframe = \
+                    self._get_optional_metadata_from_rdbms_connection(
+                        connection_args, optional_queries, base_dataframe,
+                        metadata_definition)
 
-        enrich_metadata_dict = user_config.get_enrich_metadata_dict()
+        enrich_metadata_dict = config.get_enrich_metadata_dict()
 
         if enrich_metadata_dict:
             metadata_enricher = self._get_metadata_enricher()(
@@ -195,23 +222,37 @@ class MetadataScraper:
                              on=[table_container_mame_col, table_name_col])
         return dataframe
 
+    @classmethod
+    def _get_metadata_from_csv(cls, csv_path):
+        return pd.read_csv(csv_path)
+
+    @classmethod
+    def _is_metadata_from_connection(cls, connection_args):
+        return connection_args and len(connection_args.keys()) > 0
+
+    @classmethod
+    def __resolve_base_metadata_query(cls, config, query):
+        # If user provided a different base metadata query
+        # override the default query.
+        resolved_query = query
+
+        if config and config.base_metadata_query:
+            resolved_query = config.base_metadata_query
+
+        return resolved_query
+
     # To connect to the RDBMS, it's required to override this method.
     # If you are ingesting from a CSV file, this method is not used.
     def _create_rdbms_connection(self, connection_args):
         raise NotImplementedError(
             'Implementing this method is required to connect to a RDBMS!')
 
-    @classmethod
-    def _get_metadata_from_csv(cls, csv_path):
-        return pd.read_csv(csv_path)
-
     def _get_query_assembler(self):
         raise NotImplementedError('Implementing this method is required '
                                   'to run multiple optional queries')
 
     def _get_metadata_enricher(self):
-        raise NotImplementedError('Implementing this method is required '
-                                  'to enrich metadata attributes')
+        return base_metadata_enricher.BaseMetadataEnricher
 
     def _execute_refresh_query(self, cursor, query):
         """
